/*
 * Copyright (C) 2022 Yubico.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.yubico.authenticator.yubikit

import com.yubico.yubikit.core.YubiKeyConnection
import com.yubico.yubikit.core.YubiKeyDevice
import kotlin.coroutines.suspendCoroutine

suspend inline fun <reified C : YubiKeyConnection, T> YubiKeyDevice.withConnection(
    crossinline block: (C) -> T
): T = suspendCoroutine { continuation ->
    requestConnection(C::class.java) {
        continuation.resumeWith(runCatching {
            block(it.value)
        })
    }
}
